#include <cassert>
#include <chrono>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <string>
#include <thread>
#include <vector>

#include "Configuration.hpp"
#include "MpiEnvironment.hpp"
#include "MpiWireworld.hpp"
#include "WireWorld.hpp"

using namespace std::string_literals;

int main(int argc, char* argv[]) {
	static constexpr auto DebugRank = 1;
	const auto& starttime = std::chrono::system_clock::now();

	MpiEnvironment env(argc, argv);
	const auto& cfg = Configuration::parseArgs(argc, argv, env);

	if (env.worldRank() == DebugRank) {
		std::cout << "Config: \n";
		std::cout << "col:" << cfg.GridColumns << "\n";
		std::cout << "row:" << cfg.GridRows << "\n";
	}

	MpiWireworld ww(env, cfg);

	ww.Communicate();
	if (env.worldRank() == DebugRank) { auto err = std::system("clear"); }
	if (env.worldRank() == DebugRank) std::cout << ww << '\n';
	for (std::size_t i{0}; i < 30; ++i) {
		using namespace std::chrono_literals;
		std::this_thread::sleep_for(.5s);

		ww.simulateStep();
		if (env.worldRank() == DebugRank) { auto err = std::system("clear"); }
		if (env.worldRank() == DebugRank) std::cout << ww << '\n';
	}

	if (env.isMaster()) {
		std::cout
		    << "Execution time:"
		    << std::chrono::duration<double>{std::chrono::system_clock::now() -
		                                     starttime}
		           .count()
		    << "s\n";
	}
	return EXIT_SUCCESS;
}